/* This file is part of the Multiagent Decision Process (MADP) Toolbox v0.3. 
 *
 * The majority of MADP is free software released under GNUP GPL v.3. However,
 * some of the included libraries are released under a different license. For 
 * more information, see the included COPYING file. For other information, 
 * please refer to the included README file.
 *
 * This file has been written and/or modified by the following people:
 *
 * Frans Oliehoek 
 * Matthijs Spaan 
 *
 * For contact information please see the included AUTHORS file.
 */

/* Only include this header file once. */
#ifndef _NULLPLANNERFACTORED_H_
#define _NULLPLANNERFACTORED_H_ 1

/* the include directives */
#include <iostream>
#include "Globals.h"

#include "PlanningUnitFactoredDecPOMDPDiscrete.h"

/** \brief NullPlannerFactored represents a planner which does nothing, but
 * can be used to instantiate a PlanningUnitDecFactoredPOMDPDiscrete. */
class NullPlannerFactored : 
    public PlanningUnitFactoredDecPOMDPDiscrete
{
private:    
    
protected:
    
public:

    /// (default) Constructor
    NullPlannerFactored(FactoredDecPOMDPDiscreteInterface* p);

    NullPlannerFactored(size_t horizon, FactoredDecPOMDPDiscreteInterface* p);

    NullPlannerFactored(const PlanningUnitMADPDiscreteParameters &params,
                size_t horizon, FactoredDecPOMDPDiscreteInterface* p);

    ~NullPlannerFactored(){};

    /// Only present to satisfy the interface.
    void Plan();
    /// Only present to satisfy the interface.
    double GetExpectedReward() const;
    /// Only present to satisfy the interface.
    boost::shared_ptr<JointPolicy> GetJointPolicy();

};


#endif /* !_NULLPLANNERFACTORED_H_ */

// Local Variables: ***
// mode:c++ ***
// End: ***
